#!/usr/bin/env python

__revision__ = "$Id: setup.py 24743 2004-07-21 01:42:01Z dbinger $"

from distutils.core import setup

setup(name = "Sancho",
      version = "0.3",
      description = "Unit testing framework",

      author = "CNRI, (originally Greg Ward)",
      author_email = "webmaster@mems-exchange.org",
      url = "http://www.mems-exchange.org/software/sancho/",
      license = "see LICENSE.txt",
      package_dir = {"sancho": "."},
      packages = ["sancho"],
      scripts = ["scripts/run_sancho_tests"],
     )
